	-- Declaração das Bibliotecas e pacotes utilizados
	library ieee;
	use ieee.numeric_std.all;

	-- Entidades e Mapeamento de Portas
	entity CTRL_MOTOR is
		port (
						CLOCK			:   in  bit;								-- Clock do kit de CPLD
						RESETn		:   in  bit;								-- Botão de reset do kit de CPLD
						ENABLEn		:   in  bit;								-- Pino que habilitação do motor
						SENTIDO		:   in  bit;								-- Sentido de rotação horária ou anti-horária
						MAIS, MENOS	:   in  bit;								-- Controle de velocidade
						T				:	 out bit_vector (3 downto 0)		-- Vetor de saída que vai para o motor
				);
	end entity CTRL_MOTOR;
	
	-- Sinal interno do projeto, onde divide-se o 
	-- ciclo de clocks, atuando como uma escala.
	architecture LOGICA of CTRL_MOTOR is
		signal PRESCALER: integer range 0 to 15 := 10;					
		
			begin
			
	 			controle:
				process (CLOCK, RESETn, SENTIDO, ENABLEn, MAIS, MENOS)
				   -- Declaração de variáveis
					variable conta:  			integer range 0 to 63 := 0;		   -- Conta servirá para vereficar os botões do
																									-- controle de velocidade, começando em "0".
					variable POS_MOTOR:  	bit_vector (3 downto 0):= "0001";	-- Posição do Motor começando em 0001
					variable contagem:  		integer range 0 to 15 := 10;			-- Contagem conta o número de pulsos, começando 
																									-- em "10" (mesmo de PRESCALER) 
						
				begin
               if RESETn = '0' then				 -- Se RESETn está ativado, então
						conta:= 0;						 -- as variáveis ficam no estado inicial,
						PRESCALER <= 10;				 -- ou seja, conta em 0, PRESCALER em 10,
						contagem := PRESCALER;	    -- contagem recebe o valor de PRESCARLER,
						POS_MOTOR := "0001";			 -- e POS_MOTOR em 0001.
				
					-- Se RESETn for 1, e um evento de subida de clock e ENABLE em 0, então
               elsif (CLOCK'event and CLOCK = '1' and ENABLEn = '0') then
					
						
						if conta /= 32 then				-- Se conta for diferente de 32, então
							conta := conta + 1;			-- acrescentamos 1 no "conta".
							
						else 																	-- Se conta for igual a 32,
							conta := 0;														-- atribuímos "0" em conta...
							
							if (MAIS = '0') and (PRESCALER > 1) then				-- Se MAIS estiver ativo e PRESCALER for maior que um,
								PRESCALER <= PRESCALER - 1;					 		-- diminui-se um do PRESCALER, ou seja, diminui-se a divisão
																								-- para cada pulso de clock, aumentando a velocidade do motor.
			
							elsif (MENOS = '0') and (PRESCALER < 15) then		-- Se MENOS estiver ativo e PRESCALER for menor que quinze,
								PRESCALER <= PRESCALER + 1;					 		-- soma-se um ao PRESCALER, ou seja, aumenta-se a divisao
																								-- para cada pulso de clock, diminuindo a velocidade do motor.
						   end if;
						end if;
				 
						if contagem /= 0 then							-- Se contagem for diferente de 0, então
							contagem := contagem - 1;					-- diminui-se 1 na contagem.
						
						else													-- Se contagem for 0,
							contagem := PRESCALER;						-- ela receberá o valor de PRESCALER e...
							if (SENTIDO = '0') then						-- Se SENTIDO estiver ativo, então
								 POS_MOTOR := POS_MOTOR rol 1; 		-- rotacionamos os bits de POS_MOTOR para esquerda.
								 
							else												-- Se SENTIDO não estiver ativo, então
								 POS_MOTOR := POS_MOTOR  ror 1; 		-- rotacionamos os bits de POS_MOTOR para direita.
							 
							end if;            
						end if;
					 end if;
                    
					T <= POS_MOTOR;		-- Bits de POS_MOTOR vão para T (saida para o motor)
            
				end process;    
   end architecture LOGICA;