LIBRARY ieee;
USE ieee.std_logic_1164.all; 

LIBRARY work;

entity tb_motores is
end entity;

architecture simulacao of tb_motores is

-- Aqui serão declarados todos os sinais utilizados na simulação.
-- Os sinais não precisam ter os mesmos nomes que tem no projeto.
--	Normalmente cria-se um sinal para cada porta da entity.

		signal 	CLOCK, RESETn,	ENABLEn, SENTIDO, MAIS, MENOS :    bit;
		signal   T :    bit_vector (3 downto 0);

begin

UUT: entity work.ctrl_motor -- Após 'work.', vai o nome da entidade a ser testada.

-- Aqui serão atribuídos os pinos. Eles são atribuídos em ordem; por exemplo, no projeto a primeira porta 
--	declarada na entity é o CLOCK e a variável desejada para comandá-la é CLOCK.
-- Portanto o primeiro 'signal' no 'port map' será CLOCK. Os signals são separados por vírgulas.

port map
(
		CLOCK, RESETn,           
      ENABLEn,           	    
      SENTIDO,
		MAIS, MENOS,
		T          
);


	GERA_CLOCK:		-- Este processo irá gerar pulsos 
		process		-- de CLOCK para a entrada CLOCK.
		begin
			for cont in 0 to 140 loop
				CLOCK <= '1';
				wait for 5 ms;
				CLOCK <= '0';
				wait for 5 ms;
				end loop;	
				wait;
		end process GERA_CLOCK;


	SIMULACAO:		-- Este processo realiza a 
		process		-- simulação em cada etapa.
		begin

				RESETn  <= '1';
				ENABLEn <= '1';				-- Neste período o motor está desativado,
				SENTIDO <= '1';				-- pois ENABLEn não está ativo.
				MAIS 	  <= '1';
				MENOS   <= '1';
				wait for 25 ms;				
			
				ENABLEn <= '0';				-- Em seguida ativa-se a entrada ENABLEn para
				wait for 25 ms;				-- habilitar o controle do motor de passo.
													-- Com ele já ativo, o motor funciona com
													-- a velocidade selecionada pelas entradas
													-- MAIS e MENOS.
					
				MENOS   <= '0';				-- Neste período dimiuiu-se a velocidade
				wait for 100 ms;				-- do motor, ou seja, foi aumentado o PRESCALER.
													-- Espera-se o tempo necessário para notar alguma
													-- alteração na velocidade durante a simulação.
			
				MENOS   <= '1';				-- Aqui desativou-se a entrada MENOS, deixando a
				wait for 300 ms;				-- velocidade do motor constante.
		
				MAIS    <= '0';				-- Neste processo aumentou-se a velocidade
				wait for 100 ms;				-- do motor, ou seja, foi diminuido o PRESCALER.
													-- Também espera-se o tempo necessário para
													-- notar uma diferença na alteração de velocidade.
																					
				MAIS    <= '1';				-- Foi desativado a entrada MAIS, possibilitando
				wait for 300 ms;				-- a velocidade do motor constante.
				
				SENTIDO <= '0';				-- Ativou-se a entrada SENTIDO, acarretando no 
				wait for 400 ms;				-- rotacionamento de bits da saída para a esquerda
				
				RESETn  <= '0';				--	Por último, o circuito foi resetado, voltando
				wait for 10 ms;				-- ao seu estado inicial.
			
				wait;
			
		end process SIMULACAO;

end architecture;